@@ -6,5 +6,5 @@
 @file:__init__.py.py
 @time:2021/03/05
 """
-from .reader import read_gef, read_gem, read_ann_h5ad, read_stereo_h5ad, anndata_to_stereo, stereo_to_anndata, read_gef_info
+from .reader import read_gef, read_gem, read_ann_h5ad, read_stereo_h5ad, anndata_to_stereo, stereo_to_anndata, read_gef_info, read_seurat_h5ad
 from .writer import write, write_h5ad, write_mid_gef, update_gef
