@@ -11,5 +11,12 @@ from .scatter import base_scatter
 from .marker_genes import marker_genes_text, marker_genes_heatmap
 from .plot_collection import PlotCollection
 from .interact_plot.spatial_cluster import interact_spatial_cluster
+from .interact_plot.annotation_cluster import interact_spatial_cluster_annotation
 from .interact_plot.interactive_scatter import InteractiveScatter
 from .interact_plot.poly_selection import PolySelection
+from .ms_plot import MSPlot
+from .plot_cluster_traj import PlotClusterTraj
+from .plot_vec import PlotVec
+from .plot_cluster_traj_3d import PlotClusterTraj3D
+from .plot_vec_3d import PlotVec3D
+from .vt3d_browser.example import Plot3DBrowser
