@@ -19,15 +19,14 @@ from typing import Optional
 import holoviews.operation.datashader as hd
 from stereo.log_manager import logger
 import copy
-from stereo.config import StereoConfig
+from stereo.stereo_config import stereo_conf
 from stereo.tools.boundary import ConcaveHull
 
-conf = StereoConfig()
 
 link = link_selections.instance()
 pn.param.ParamMethod.loading_indicator = True
 
-colormaps = conf.linear_colormaps
+colormaps = stereo_conf.linear_colormaps
 
 
 class InteractiveScatter:
@@ -47,14 +46,19 @@ class InteractiveScatter:
         self.height = height
         self.bgcolor = bgcolor
         # self.link = link_selections.instance()
+        if self.data.cells.total_counts is None:
+            total_counts = self.data.exp_matrix.sum(axis=1).T.A[0] if self.data.issparse() else self.data.exp_matrix.sum(axis=1).T
+        else:
+            total_counts = self.data.cells.total_counts
         self.scatter_df = pd.DataFrame({
             # 'cell': self.data.cell_names,
             'x': self.data.position[:, 0],
             'y': self.data.position[:, 1] * -1,
             # 'count': np.array(self.data.exp_matrix.sum(axis=1))[:, 0],
-            'count': np.array(self.data.exp_matrix.sum(axis=1))[:,
-                     0] if self.data.cells.total_counts is None else self.data.cells.total_counts
-        }).reset_index()
+            # 'count': np.array(self.data.exp_matrix.sum(axis=1))[:, 0] if self.data.cells.total_counts is None else self.data.cells.total_counts
+            'count': total_counts
+        })
+        self.scatter_df.reset_index(inplace=True)
         self.selected_exp_data = None
         self.drop_checkbox = pn.widgets.Select(
             name='method',
@@ -133,6 +137,42 @@ class InteractiveScatter:
             cg.create_Region_Bgef(origin_file_path, output_path, coors)
         return output_path
 
+    @param.depends(link.param.selection_expr)
+    def get_selected_boundary_coors(self) -> list:
+        """
+        get selected area exp boundary coords, list contains each x,y
+        Returns:
+
+        """
+        selected_pos = hv.Dataset(self.scatter_df).select(link.selection_expr).data.index
+        self.generate_selected_expr_matrix(selected_pos, self.drop_checkbox.value)
+        exp_matrix_data = self.selected_exp_data.position.tolist()
+        init = ConcaveHull(exp_matrix_data, 3)
+        concave_hull = init.calculate().tolist()
+        concave_hull = [int(i) for k in concave_hull for i in k]
+        self.list_poly_selection_exp_coors.append(concave_hull)
+        return self.list_poly_selection_exp_coors
+
+    def export_high_res_area(self, origin_file_path: str, output_path: str, cgef: bool = False) -> str:
+        """
+        export selected area in high resolution
+        Args:
+            origin_file_path: origin file path which you read
+            output_path: location the high res file storaged
+            cgef: bool, default False, set True if read in cellbin
+        Returns:
+            output_path
+        """
+        coors = self.get_selected_boundary_coors()
+
+        from gefpy.cgef_adjust_cy import CgefAdjust
+        cg = CgefAdjust()
+        if cgef:
+            cg.create_Region_Cgef(origin_file_path, output_path, coors)
+        else:
+            cg.create_Region_Bgef(origin_file_path, output_path, coors)
+        return output_path
+
     def interact_scatter(self):
         pn.extension()
         hv.extension('bokeh')
